package com.fjhx.account.controller.account;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fjhx.account.entity.account.dto.AccountDeptRunningWaterDto;
import com.fjhx.account.entity.account.dto.AccountDeptRunningWaterSelectDto;
import com.fjhx.account.entity.account.dto.DeptExpenseDto;
import com.fjhx.account.entity.account.po.AccountDeptRunningWater;
import com.fjhx.account.entity.account.vo.DeptExpenseVo;
import com.fjhx.account.service.account.AccountDeptRunningWaterService;
import com.fjhx.common.constant.SourceConstant;
import com.fjhx.item.util.excel.util.ExcelUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.util.List;
import java.util.Map;


/**
 * <p>
 * 部门-资金流水关联表 前端控制器
 * </p>
 *
 * @author
 * @since 2023-04-13
 */
@DS(SourceConstant.ACCOUNT)
@RestController
@RequestMapping("/accountDeptRunningWater")
public class AccountDeptRunningWaterController {

    @Autowired
    private HttpServletResponse response;

    @Autowired
    private AccountDeptRunningWaterService accountDeptRunningWaterService;

    /**
     * 部门-资金流水关联表分页
     */
    @PostMapping("/page")
    public Page<DeptExpenseVo> page(@RequestBody DeptExpenseDto dto) {
        return accountDeptRunningWaterService.getPage(dto);
    }

    /**
     * 部门-资金流水关联表分页
     */
    @PostMapping("/export")
    public void export(@RequestBody DeptExpenseDto dto) {
        dto.setPageNum(1);
        dto.setPageSize(Integer.MAX_VALUE);
        Page<DeptExpenseVo> page = accountDeptRunningWaterService.getPage(dto);
        ExcelUtil.export(response, page.getRecords(), DeptExpenseVo.class);
    }

    @PostMapping("/totality")
    public BigDecimal totality(@RequestBody DeptExpenseDto dto) {
        dto.setPageNum(1);
        dto.setPageSize(Integer.MAX_VALUE);
        Page<DeptExpenseVo> page = accountDeptRunningWaterService.getPage(dto);
        List<DeptExpenseVo> records = page.getRecords();
        if (records.isEmpty()) {
            return BigDecimal.ZERO;
        }

        return records.stream().map(DeptExpenseVo::getAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
    }


    /**
     * 部门-资金流水关联表新增
     */
    @PostMapping("/add")
    public void add(@RequestBody AccountDeptRunningWaterDto accountDeptRunningWaterDto) {
        accountDeptRunningWaterService.add(accountDeptRunningWaterDto);
    }

    /**
     * 部门-资金流水关联表新增
     */
    @PostMapping("/bathAdd")
    public void bathAdd(@RequestBody List<AccountDeptRunningWater> list) {
        accountDeptRunningWaterService.saveBatch(list);
    }

    /**
     * 部门-资金流水报表
     *
     * @param dto
     * @return
     */
    @PostMapping("/DeptRunningWaterReport")
    public List<Map<String, Object>> DeptRunningWaterReport(@RequestBody DeptExpenseDto dto) {
        return accountDeptRunningWaterService.DeptRunningWaterReport(dto);
    }

    /**
     * 部门-资金流水统计（ 部门-资金流水列表）
     *
     * @param dto
     * @return
     */
    @PostMapping("/deptRunningWaterStatistics")
    public Map<String, Object> deptRunningWaterStatistics(@RequestBody AccountDeptRunningWaterSelectDto dto) {
        return accountDeptRunningWaterService.deptRunningWaterStatistics(dto);
    }


}
