package com.fjhx.account.service.account.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.dynamic.datasource.toolkit.DynamicDataSourceContextHolder;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.account.controller.utils.DateUtils;
import com.fjhx.account.entity.account.dto.AccountDeptRunningWaterDto;
import com.fjhx.account.entity.account.dto.AccountDeptRunningWaterSelectDto;
import com.fjhx.account.entity.account.dto.DeptExpenseDto;
import com.fjhx.account.entity.account.po.AccountDeptRunningWater;
import com.fjhx.account.entity.account.vo.AccountDeptRunningWaterVo;
import com.fjhx.account.entity.account.vo.AccountRunningWaterVo;
import com.fjhx.account.entity.account.vo.DeptExpenseVo;
import com.fjhx.account.mapper.account.AccountDeptRunningWaterMapper;
import com.fjhx.account.service.account.AccountDeptRunningWaterService;
import com.fjhx.common.constant.SourceConstant;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.system.service.ISysDeptService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;


/**
 * <p>
 * 部门-资金流水关联表 服务实现类
 * </p>
 *
 * @author
 * @since 2023-04-13
 */
@Service
public class AccountDeptRunningWaterServiceImpl extends ServiceImpl<AccountDeptRunningWaterMapper, AccountDeptRunningWater> implements AccountDeptRunningWaterService {

    @Autowired
    private ISysDeptService sysDeptService;

    private static final Set<Long> deptIds = new HashSet<Long>() {{
        add(1508293663904624641L);
        add(1508293745521586177L);
        add(1508351674429861890L);
        add(1509835006418219009L);
        add(1545289616599367681L);
        add(1567825087275495425L);
        add(1663020305968291842L);
    }};

    @Override
    public Page<DeptExpenseVo> getPage(DeptExpenseDto dto) {
        dto.setDeptIds(deptIds);
        return baseMapper.getPage(dto.getPage(), dto);
    }

    @Override
    public AccountDeptRunningWaterVo detail(Long id) {
        AccountDeptRunningWater AccountDeptRunningWater = this.getById(id);
        AccountDeptRunningWaterVo result = BeanUtil.toBean(AccountDeptRunningWater, AccountDeptRunningWaterVo.class);
        return result;
    }

    /**
     * 部门-资金流水关联表新增
     */
    @Override
    public void add(AccountDeptRunningWaterDto accountDeptRunningWaterDto) {
        if (ObjectUtil.isNotEmpty(accountDeptRunningWaterDto.getId())) {
            this.updateById(accountDeptRunningWaterDto);
        } else {
            this.save(accountDeptRunningWaterDto);
        }
    }

    @Override
    public void edit(AccountDeptRunningWaterDto accountDeptRunningWaterDto) {
        this.updateById(accountDeptRunningWaterDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    /**
     * 部门-资金流水报表
     *
     * @param dto
     * @return
     */
    @Override
    public List<Map<String, Object>> DeptRunningWaterReport(DeptExpenseDto dto) {
        if (ObjectUtil.isEmpty(dto.getBeginTime()) && ObjectUtil.isEmpty(dto.getEndTime())) {
            throw new ServiceException("查询的时间范围不能为空！");
        }

        dto.setDeptIds(deptIds);
        dto.setPageNum(1);
        dto.setPageSize(Integer.MAX_VALUE);
        Page<DeptExpenseVo> page = baseMapper.getPage(dto.getPage(), dto);
        List<DeptExpenseVo> records = page.getRecords();
        if (records.isEmpty()) {
            return Collections.emptyList();
        }

        //获取月份集合
        List<String> months = DateUtils.getMonthBetweenDate(dto.getBeginTime(), dto.getEndTime());

        Map<Long, List<DeptExpenseVo>> map = records.stream().collect(Collectors.groupingBy(DeptExpenseVo::getDepartmentId));

        List<Map<String, Object>> list = new ArrayList<>();
        map.forEach((k, v) -> {
            Map<String, Object> item = new HashMap<>();
            Map<String, List<DeptExpenseVo>> collect = v.stream().collect(Collectors.groupingBy(DeptExpenseVo::getMonth));
            item.put("deptName", v.get(0).getDeptName());
            item.put("accountRunningWaterVoList", getList(months, collect));
            list.add(item);
        });

        return list;
    }

    private List<Map<String, Object>> getList(List<String> months, Map<String, List<DeptExpenseVo>> collect) {
        return months.stream().map(item -> {
            // 收入金额
            BigDecimal incomeAmount = BigDecimal.ZERO;

            // 支出金额
            BigDecimal expenditureAmount = BigDecimal.ZERO;

            List<DeptExpenseVo> list = collect.getOrDefault(item, Collections.emptyList());

            for (DeptExpenseVo deptExpenseVo : list) {
                BigDecimal amount = deptExpenseVo.getAmount();
                if (amount.compareTo(BigDecimal.ZERO) > 0) {
                    incomeAmount = incomeAmount.add(amount);
                }else {
                    expenditureAmount = expenditureAmount.add(amount);
                }
            }

            Map<String, Object> map = new HashMap<>();
            map.put("time", item);
            map.put("incomeAmount", incomeAmount);
            map.put("expenditureAmount", BigDecimal.ZERO.subtract(expenditureAmount));
            map.put("balanceAmount", incomeAmount.add(expenditureAmount));

            return map;
        }).collect(Collectors.toList());
    }

    /**
     * 部门-资金流水统计（ 部门-资金流水列表）
     *
     * @param dto
     * @return
     */
    @Override
    public Map<String, Object> deptRunningWaterStatistics(AccountDeptRunningWaterSelectDto dto) {
        //存放流水统计数据
        Map<String, Object> map = new HashMap<>();

        QueryWrapper<Object> query = Wrappers.query();
        query.groupBy("rwd.dept_id");
        //查询部门-资金流水统计信息
        List<AccountRunningWaterVo> accountRunningWaterVoList = baseMapper.deptRunningWaterStatistics(query);

        //计算收入合计数据
        BigDecimal incomeAmount = accountRunningWaterVoList.stream().map(accountRunningWaterVo ->
                accountRunningWaterVo.getIncomeAmount()).reduce(BigDecimal.ZERO, BigDecimal::add);

        //计算支出合计数据
        BigDecimal expenditureAmount = accountRunningWaterVoList.stream().map(accountRunningWaterVo ->
                accountRunningWaterVo.getExpenditureAmount()).reduce(BigDecimal.ZERO, BigDecimal::add);
        map.put("incomeAmount", incomeAmount);
        map.put("expenditureAmount", expenditureAmount);

        //查询部门信息
        //切换数据源
        DynamicDataSourceContextHolder.push(SourceConstant.BASE);
        List<SysDept> sysDeptList = sysDeptService.list();
        Map<Long, List<SysDept>> sysDeptMap = sysDeptList.stream().collect(Collectors.groupingBy(SysDept::getDeptId));

        //赋值部门信息
        for (AccountRunningWaterVo accountRunningWaterVo : accountRunningWaterVoList) {
            List<SysDept> sysDepts = sysDeptMap.get(accountRunningWaterVo.getDeptId());
            if (CollectionUtils.isNotEmpty(sysDepts)) {
                accountRunningWaterVo.setDeptName(sysDepts.get(0).getDeptName());
            }
        }
        map.put("accountRunningWaterVoList", accountRunningWaterVoList);
        return map;
    }

}
